
// Author Zecronious
// Date 31/08/2013
//-------------------

// The New World
//-------------------
// Container for everything in the game

World

	var
	// Defaults
	//------------------
		Area/defaultLocation // Set in New()
		list/areas = list() // Contains all areas
		list/areaNames = list() // Contains the string names of all areas

	// Time Keeping
	//------------------
		numberOfTicks = 0
		numberOfSeconds = 0
		numberOfMinutes = 0
		numberOfHours = 0
		numberOfDays = 0
		// numberOfYears = 0

	// Constants
	//-------------------
		TICK_DELAY = 15  // Set to the tick value you want
		SECOND_DELAY = 10 // This is accurate for 10 milliseconds by default

	// Engage world
	//-------------------
	New()

		Start_Loop()
		Setup_Areas()
		Setup_Area_Names()

	proc

	// Setup All Areas
	//-------------------
		Setup_Areas()

			// The first area listed is the default location everyone is sent to at the start
			areas.Add(new/Area/Darkwood_Forest)

			// All these areas including the first ^ are unlocked at the beginning.
			areas.Add(new/Area/Forgotten_Tower)

			// To add unlockable areas add the areas to the avatar's list of unlocked areas.

		Setup_Area_Names()

			for(var/Area/A in areas)
				areaNames.Add(A.name)

	// Main Game Loop
	//-------------------

		// Absolute parent loop
		Start_Loop()

			// Jump over spawn, execute code in spawn in background
			spawn()

				// Asynchronous clock
				while(src)
					sleep(TICK_DELAY)
					Increment_Ticks()
					Call_Actions() // Call all players avatars to act

			spawn()

				// Seconds govern all units of server time except ticks
				while(src)
					sleep(SECOND_DELAY)
					Increment_Seconds()

					if(numberOfSeconds != 0 && numberOfSeconds % 60 == 0) // Only update for increments of 60
						Evaluate_Time()

					Refresh_Player_Title()

	// Time Keeping
	//-------------------

		// Keep Time
		//-------------------
		Evaluate_Time()

			numberOfMinutes = numberOfSeconds / 60

			if(numberOfMinutes != 0 && numberOfMinutes % 60 == 0)
				numberOfHours = numberOfMinutes / 60

			if(numberOfHours != 0 && numberOfHours % 24 == 0)
				numberOfDays = numberOfHours / 24

		// Increment Timers
		//-------------------
		Increment_Seconds()
			numberOfSeconds ++

		Increment_Ticks()
			numberOfTicks ++

// Player Managment
//-------------------

var/global
	playersOnline = 0
	list/playerList = list()

proc
	// Calls the Action() method of all avatars
	Call_Actions()

		for(var/client/C)
			if(C.mob.avatar != null)
				C.mob.avatar.Action()


	// Refreshes the lists that each player has
	Refresh_Player_Lists()

		for(var/client/C)
			if(C.mob)
				C.mob.Update_Player_List()

	// Refreshes the player's titlebar with a new time
	Refresh_Player_Title()

		for(var/client/C)
			if(C.mob)
				C.mob.Set_Title_Bar("[world.name] \[[theWorld.numberOfSeconds]:[theWorld.numberOfMinutes]:[theWorld.numberOfHours]\]")

	// Adds a new player to the list, counts up
	Add_Player(name)

		playersOnline ++
		playerList.Add(name)
		Refresh_Player_Lists()

	// Removes a player from the list, counts down
	Remove_Player(name)

		playersOnline --
		playerList.Remove(name)
		Refresh_Player_Lists()

// Global Variables
//-------------------
var/global
	// The actual world
	World/theWorld

// Original World Setup
//-------------------
// Byond has basic world functionality. This code extends it, using
// the custom World object defined above

world
	// Constructor
	New()
		// Do all automatic processing
		..()

		// Make the actual world
		theWorld = new/World
